//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Drawing;

using Brush = MindFusion.Drawing.Brush;


namespace MindFusion.Diagramming.WinForms.Samples.CS.FractalLayout
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

		private void MainForm_Load(object sender, EventArgs e)
		{
			diagram.LinkHeadShape = null;
			diagram.DefaultShape = Shapes.Ellipse;
            diagram.DiagramLinkStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(97, 106, 127));
            diagram.DiagramLinkStyle.HeadStroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(97, 106, 127));

			diagramView.MouseWheelAction = MouseWheelAction.Zoom;
			diagramView.Behavior = Behavior.Pan;

			// use differect brush for each tree level
			brushes = new Brush[]
			{
				new PathGradientBrush(Color.FromArgb( 	156, 170, 198), Color.FromArgb(0, 0, 99)),
				new PathGradientBrush(Color.FromArgb(224, 233, 233), Color.FromArgb(192, 192, 192)),
				new PathGradientBrush(Color.FromArgb(224, 233, 233), Color.FromArgb(102, 154, 204)),
				new PathGradientBrush(Color.White, Color.FromArgb(206, 0, 0))
			};
		}

        private void OnRandomTree(object sender, EventArgs e)
        {
			diagram.ClearAll();

			ShapeNode root = diagram.Factory.CreateShapeNode(nodeBounds);
			RandomTree(root, 5, 4);
			Arrange(root);

			// the focused control also receives MouseWheel events
			diagramView.Focus();
        }

		private void Arrange(DiagramNode root)
		{
			MindFusion.Diagramming.Layout.FractalLayout layout =
				new MindFusion.Diagramming.Layout.FractalLayout();
			layout.Root = root;
			layout.Arrange(diagram);

			diagram.ResizeToFitItems(5);
			diagramView.ZoomToFit();
		}

		void RandomTree(DiagramNode node, int depth, int minChildren)
		{
			if (depth <= 0)
				return;

			Diagram diagram = node.Parent;
			int children = random.Next(depth) - 1 + minChildren;

			if (diagram.Nodes.Count < 3 && children < 2)
				children = 2;

			for (int i = 0; i < children; ++i)
			{
				// create child node and link
				ShapeNode child = diagram.Factory.CreateShapeNode(nodeBounds);
				child.Brush = brushes[depth % brushes.Length];
				diagram.Factory.CreateDiagramLink(node, child);

				// build child branch
				RandomTree(child, depth - 1, minChildren);
			}
		}

		Random random = new Random();
		RectangleF nodeBounds = new RectangleF(0, 0, 20, 20);
		Brush[] brushes;
    }
}
